/*
* Copyright 2004,2005 The Apache Software Foundation.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.apache.axis2.transport.jms;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

/**
 * This is singleton class where we keep the store of InitialContextes
 */

public class InitialContextStore {
    private static final Log log = LogFactory.getLog(InitialContextStore.class);

    /**
     * Do we need the caching of InitialContexts ?
     */
    private static boolean cachingRequired = true;

    /**
     * The singleton object
     */
    private static final InitialContextStore INSTANCE = new InitialContextStore();

    /**
     * The context store where we cache the contextes
     */
    private Map<String, InitialContext> contextStore = new HashMap<String, InitialContext>();

    /**
     * The private constructor
     */
    private InitialContextStore() {

    }

    public static boolean isCachingRequired() {
        return cachingRequired;
    }

    public void setCachingRequired(boolean cachingReauired) {
        InitialContextStore.cachingRequired = cachingReauired;
    }

    /**
     * Remove a context from the context cache
     *
     * @param epr the endpoint reference key
     */
    public static void removeContext(String epr) {
        if (cachingRequired) {
            synchronized (INSTANCE) {
                if (INSTANCE.contextStore.containsKey(epr)) {
                    if (log.isDebugEnabled()) {
                        log.debug("Remove InitialContext from cache for endpoint reference: " + epr);
                    }
                    InitialContext initialContext = INSTANCE.contextStore.remove(epr);
                    try {
                        initialContext.close();
                    } catch (NamingException e) {
                        log.error("Failed to close an InitialContext. Continue any way..", e);
                    }

                }
            }
        }
    }

    /**
     * Returns a context. If this context already avilable returns it.
     *
     * @param epr        the EPR which is the key for the cache
     * @param properties initial context properties
     * @return InitialContext instance
     * @throws NamingException The naming exception
     */
    public InitialContext getContext(String epr, Hashtable<String, String> properties) throws NamingException {
        InitialContext context = null;
        if (cachingRequired) {
            // This bit has to be synchoronized otherwise if you have two threads
            // go through here at the same time, two InitialContext's (of javax.naming.InitialContext)
            // will be created. Not necessarily bad but if you have lots of different epr's you'll get
            // lots of InitialContext's created unnecesarily and as these are heavy objects that's worse than
            // having lock which in the grand scheme of things is insignificant.
            synchronized (INSTANCE) {
                if (!INSTANCE.contextStore.containsKey(epr)) {
                    if (log.isDebugEnabled()) {
                        log.debug("Creating an initial Context for endpoint refernce: " + epr);
                    }
                    context = new SynchronizedInitialContext(properties);
                    INSTANCE.contextStore.put(epr, context);
                } else {
                    if (log.isDebugEnabled()) {
                        log.debug("Using a cached context for the the endpoint refernces: " + epr);
                    }
                    context = INSTANCE.contextStore.get(epr);
                }
            }

        } else {
            context = new InitialContext(properties);
        }
        return context;
    }

    public static InitialContextStore getInstance() {
        return INSTANCE;
    }

    /**
     * Close the context after the usage
     * @param initialContext the intial Context
     * @throws NamingException in case of an error
     */
    public static void closeContext(Context initialContext) throws NamingException {
        if (!cachingRequired) {
            try {
                initialContext.close();
            } catch (NamingException e) {
                log.error("Cloud not close the context", e);
            }
        }
    }
}
