/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.axis2.transport.jms;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.activation.DataSource;
import java.beans.XMLEncoder;
import java.io.*;

/**
 * Data source implementation wrapping a Java Object
 * <p/>
 * Underlying object can be accessed through the #getObject method
 */
public class ObjectDataSource implements DataSource {

    private static final Log log = LogFactory.getLog(ObjectDataSource.class);

    /** Underling java object */
    private Object object;
    /** Content type to be used to encode Object into different representations*/
    private String contentType;

    public ObjectDataSource(Object object, String contentType) {
        this.object = object;
        this.contentType = contentType;
    }

    /**
     * Gets as a XML encoded byte stream
     * @return an XML encoded byte stream
     * @throws IOException
     */
    public InputStream getInputStream() throws IOException {
        return new ByteArrayInputStream(serialize(object));
    }

    /**
     * This operation it not supported
     * @return throws UnsupportedOperationException
     * @throws IOException
     */
    public OutputStream getOutputStream() throws IOException {
        throw new UnsupportedOperationException();
    }

    public String getContentType() {
        return contentType;
    }

    public String getName() {
        return "java.object";
    }

    /**
     * Reruns the underlying java object
     * @return the underlying java object
     */
    public Object getObject() {
        return object;
    }

    /**
     * Serialize the given object into a textual representation
     *
     * @param data given object
     * @return an array of byte of the given object
     */
    private byte[] serialize(Object data) {

        ByteArrayOutputStream binOut = null;
        byte[] result = null;
        try {
            binOut = new ByteArrayOutputStream();
            XMLEncoder encoder = new XMLEncoder(binOut);
            encoder.writeObject(data);
            encoder.close();
            result = binOut.toByteArray();
        } finally {
            if (binOut != null) {
                try {
                    binOut.close();
                } catch (IOException ignored) {
                }
            }
        }
        return result;
    }
}
