/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.axis2.transport.jms;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.builder.Builder;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.format.DataSourceMessageBuilder;
import org.apache.axis2.transport.base.BaseConstants;
import org.apache.axis2.transport.base.BaseUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.xml.namespace.QName;
import java.io.InputStream;
import java.io.IOException;

/**
 * Message builder for java object payloads.
 * <p/>
 * This builder processes wraps the data in a wrapper element. The name of the wrapper element can
 * be configured as a service parameter (see {@link BaseConstants#WRAPPER_PARAM}).
 * It defaults to {@link BaseConstants#DEFAULT_OBJECT_WRAPPER}.
 */
public class ObjectMessageBuilder implements DataSourceMessageBuilder {

    private static final Log log = LogFactory.getLog(ObjectMessageBuilder.class);

    /**
     * To be convert the object into some other representation such as an XML Infoset
     */
    private Builder builder;

    public ObjectMessageBuilder() {
    }

    public ObjectMessageBuilder(Builder builder) {
        this.builder = builder;
    }

    /**
     * Process the input data and map into an OMElement representation
     * @param dataSource Source of the message
     * @param contentType  content type
     * @param msgContext Axis2 Message context to be used to keep the payload
     * @return  the payload corresponding to the data
     * @throws AxisFault
     */
    public OMElement processDocument(DataSource dataSource,
                                     String contentType,
                                     MessageContext msgContext) throws AxisFault {
        if (builder == null) {
            QName wrapperQName = BaseConstants.DEFAULT_OBJECT_WRAPPER;
            if (msgContext.getAxisService() != null) {
                Parameter wrapperParameter = msgContext.getAxisService().getParameter(
                        BaseConstants.WRAPPER_PARAM);
                if (wrapperParameter != null) {
                    wrapperQName = BaseUtils.getQNameFromString(wrapperParameter.getValue());
                }
            }
            OMFactory factory = OMAbstractFactory.getOMFactory();
            OMElement wrapper = factory.createOMElement(wrapperQName, null);
            DataHandler dataHandler = new DataHandler(dataSource);
            wrapper.addChild(factory.createOMText(dataHandler, true));
            return wrapper;
        } else if (builder instanceof DataSourceMessageBuilder) {
            return ((DataSourceMessageBuilder) builder).processDocument(dataSource,
                    contentType,
                    msgContext);
        } else {
            try {
                return builder.processDocument(dataSource.getInputStream(), contentType, msgContext);
            } catch (IOException e) {
                handleException("Error on getting an input stream from the object datasource",
                        e);
            }
        }
        return null;
    }

    /**
     * This operation is not supported so calling this method will
     * get <code>UnsupportedOperationException</code>
     *
     * @param inputStream    Message content as an input stream
     * @param s              content type
     * @param messageContext Axis2 Message context to be used to keep the payload
     * @return throws UnsupportedOperationException
     * @throws AxisFault
     */
    public OMElement processDocument(InputStream inputStream,
                                     String s,
                                     MessageContext messageContext) throws AxisFault {

        throw new UnsupportedOperationException("The processDocument(InputStream inputStream," +
                " String s, MessageContext messageContext) operation is not supported");
    }

    private void handleException(String msg, Exception e) throws AxisFault {
        log.error(msg, e);
        throw new AxisFault(msg, e);
    }
}
