/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.apache.axis2.transport.jms;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNode;
import org.apache.axiom.om.OMOutputFormat;
import org.apache.axiom.om.OMText;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.transport.MessageFormatter;
import org.apache.axis2.transport.base.BaseConstants;
import org.apache.axis2.transport.http.util.URLTemplatingUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.activation.DataHandler;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;

/**
 * Message Formatter that encodes java object into XML and serializes
 * Object is picked from the wrapper with name {@link BaseConstants.DEFAULT_OBJECT_WRAPPER}
 */
public class ObjectMessageFormatter implements MessageFormatter {

    private static final Log log = LogFactory.getLog(ObjectMessageFormatter.class);

    /**
     *  Returns the bytes of the Java object in the ObjectDataSource
     * @param messageContext Axis2 Message context
     * @param format OMOutputFormat
     * @return A byte array of the Java object in the ObjectDataSource
     * @throws AxisFault
     */
    public byte[] getBytes(MessageContext messageContext,
                           OMOutputFormat format) throws AxisFault {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        writeTo(messageContext, format, baos, true);
        return baos.toByteArray();
    }

    /**
     * Writes the data in the Java Object into the output stream   
     */
    public void writeTo(MessageContext messageContext,
                        OMOutputFormat format, OutputStream out,
                        boolean preserve) throws AxisFault {

        DataHandler dh = getDataHandler(messageContext.getEnvelope());
        if (dh != null) {
            try {
                dh.writeTo(out);
            } catch (IOException e) {
                handleException("Error serializing binary content of element : " +
                        BaseConstants.DEFAULT_OBJECT_WRAPPER, e);
            }
        }
    }

    public String getContentType(MessageContext messageContext,
                                 OMOutputFormat format,
                                 String soapAction) {
        String contentType = (String) messageContext.getProperty(
                Constants.Configuration.CONTENT_TYPE);
        if (contentType == null) {
            contentType = "";
        } else {
            String encoding = format.getCharSetEncoding();
            if (encoding != null) {
                contentType += "; charset=" + encoding;
            }
        }
        return contentType;
    }

    public URL getTargetAddress(MessageContext messageContext,
                                OMOutputFormat format,
                                URL targetURL) throws AxisFault {
        return URLTemplatingUtil.getTemplatedURL(targetURL, messageContext, false);
    }

    public String formatSOAPAction(MessageContext messageContext,
                                   OMOutputFormat format,
                                   String soapAction) {
        return soapAction;
    }

    private void handleException(String msg, Exception e) throws AxisFault {
        log.error(msg, e);
        throw new AxisFault(msg, e);
    }

    /**
     * Extracts the <code>DataHandler</code> from the provided SOAP Message
     *
     * @param soapEnvelope the SOAP Message to be used to find the  <code>DataHandler</code>
     * @return <code>DataHandler</code> if there is one wrapped with the
     * @{code BaseConstants.DEFAULT_OBJECT_WRAPPER}. Otherwise , null
     */
    private DataHandler getDataHandler(SOAPEnvelope soapEnvelope) {
        OMElement wrapper = soapEnvelope.getBody().getFirstChildWithName(
                BaseConstants.DEFAULT_OBJECT_WRAPPER);
        OMNode omNode = wrapper.getFirstOMChild();
        if (omNode != null && omNode instanceof OMText) {
            Object dh = ((OMText) omNode).getDataHandler();
            if (dh != null && dh instanceof DataHandler) {
                return (DataHandler) dh;
            }
        }
        return null;
    }
}
